@@ -1,6 +1,10 @@
 use crate::error::Result;
 use deadpool::managed::Object;
-use deadpool_postgres::{Manager, Transaction as PoolTransaction};
+#[cfg(feature = "with-tokio-0_2")]
+use deadpool_postgres::ClientWrapper;
+#[cfg(feature = "with-tokio-1_x")]
+use deadpool_postgres::Manager;
+use deadpool_postgres::Transaction as PoolTransaction;
 use tokio_postgres::types::ToSql;
 use tokio_postgres::{Client, NoTls, ToStatement, Transaction as ClientTransaction};
 
@@ -21,6 +25,9 @@ pub async fn connect<'a>() -> Result<Connection<'a>> {
 pub enum Connection<'a> {
     ClientConnection(Client),
     ClientTransaction(ClientTransaction<'a>),
+    #[cfg(feature = "with-tokio-0_2")]
+    PoolConnection(Object<ClientWrapper, tokio_postgres::Error>),
+    #[cfg(feature = "with-tokio-1_x")]
     PoolConnection(Object<Manager>),
     PoolTransaction(PoolTransaction<'a>),
 }
