@@ -3,7 +3,6 @@ use belongs_to::{make_belongs_to, BelongsTo};
 use config::Config;
 use has_many::{make_has_many, HasMany};
 use has_one::{make_has_one, HasOne};
-use inflector::Inflector;
 use log::debug;
 use order::order_part;
 use proc_macro2::{Ident, TokenStream};
@@ -24,6 +23,9 @@ mod order;
 
 pub fn define_ar(dir: PathBuf, configs: Vec<Config>) -> Result<()> {
     let _ = env_logger::builder().is_test(true).try_init();
+    #[cfg(feature = "with-tokio-0_2")]
+    let mut rt = Runtime::new()?;
+    #[cfg(feature = "with-tokio-1_x")]
     let rt = Runtime::new()?;
 
     let columns_map: HashMap<String, Vec<Column>> = rt.block_on(async {
@@ -43,7 +45,7 @@ pub fn define_ar(dir: PathBuf, configs: Vec<Config>) -> Result<()> {
             TokenStream,
             TokenStream,
             HashMap<String, TokenStream>,
-        ) = define_ar_impl(config, &columns_map).unwrap();
+        ) = define_ar_impl(config, &configs, &columns_map).unwrap();
         for (key, val) in output_enums {
             enums.insert(key, val);
         }
@@ -149,8 +151,12 @@ ORDER BY ordinal_position
 
 fn define_ar_impl(
     config: &Config,
+    configs: &Vec<Config>,
     columns_map: &HashMap<String, Vec<Column>>,
 ) -> Result<(TokenStream, TokenStream, HashMap<String, TokenStream>)> {
+    #[cfg(feature = "with-tokio-0_2")]
+    let mut rt = Runtime::new()?;
+    #[cfg(feature = "with-tokio-1_x")]
     let rt = Runtime::new()?;
 
     rt.block_on(async {
@@ -182,10 +188,6 @@ fn define_ar_impl(
         let struct_ident: Ident = format_ident!("{}", &config.struct_name);
         let new_struct_ident: Ident = format_ident!("{}New", struct_ident);
         let builder_ident: Ident = format_ident!("{}Builder", struct_ident);
-        let builder_columns: Vec<Ident> = columns
-            .iter()
-            .map(|column| format_ident!("{}_{}", &builder_ident, &column.name))
-            .collect();
 
         let fn_delete: TokenStream = make_fn_delete(&table_name, &columns);
         let fn_insert: TokenStream = make_fn_insert(&struct_ident, &table_name, &columns);
@@ -210,7 +212,7 @@ fn define_ar_impl(
             has_many_filters_impl,
             has_many_join,
             has_many_preload,
-        } = make_has_many(config, &builder_ident, columns_map);
+        } = make_has_many(config, &builder_ident, columns_map, configs);
 
         let HasOne {
             has_one_use_plain,
@@ -223,7 +225,7 @@ fn define_ar_impl(
             has_one_filters_impl,
             has_one_join,
             has_one_preload,
-        } = make_has_one(config, &builder_ident, columns_map);
+        } = make_has_one(config, &builder_ident, columns_map, configs);
 
         let BelongsTo {
             belongs_to_use_plain,
@@ -236,7 +238,7 @@ fn define_ar_impl(
             belongs_to_filters_impl,
             belongs_to_join,
             belongs_to_preload,
-        } = make_belongs_to(config, &builder_ident, &columns);
+        } = make_belongs_to(config, &builder_ident, &columns, configs);
         let use_plain = uniq_use(has_many_use_plain, has_one_use_plain, belongs_to_use_plain);
         let use_impl = uniq_use(has_many_use_impl, has_one_use_impl, belongs_to_use_impl);
 
@@ -323,10 +325,27 @@ fn define_ar_impl(
                 #(#belongs_to_builder_field)*
             }
 
+            impl BuilderAccessor for #builder_ident {
+                fn from(x: &Self) -> &String {
+                    &x.from
+                }
+                fn table_name_as(x: &Self) -> &Option<String> {
+                    &x.table_name_as
+                }
+                fn filters(x: &mut Self) -> &mut Vec<Filter> {
+                    &mut x.filters
+                }
+                fn preload(x: &Self) -> bool {
+                    x.preload
+                }
+            }
+
             impl #builder_ident {
-                #(pub fn #column_names(&self) -> #builder_columns {
-                    #builder_columns {
-                        builder: self.clone()
+                #(pub fn #column_names(&self) -> FilterBuilder<#builder_ident, #rust_types> {
+                    FilterBuilder::<#builder_ident, #rust_types> {
+                        column_name: stringify!(#column_names),
+                        builder: self.clone(),
+                        value_type: std::marker::PhantomData
                     }
                 })*
                 #(#has_many_builder_impl)*
@@ -494,165 +513,6 @@ fn define_ar_impl(
                 }
             }
 
-            #(
-                #[allow(non_camel_case_types)]
-                pub struct #builder_columns {
-                    pub builder: #builder_ident,
-                }
-                impl #builder_columns {
-                    pub fn eq(&self, value: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(value)],
-                            operator: "=",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn gt(&self, value: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(value)],
-                            operator: ">",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn lt(&self, value: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(value)],
-                            operator: "<",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn gte(&self, value: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(value)],
-                            operator: ">=",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn lte(&self, value: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(value)],
-                            operator: "<=",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn not_eq(&self, value: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(value)],
-                            operator: "<>",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn is_null(&self) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![],
-                            operator: "IS NULL",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn is_not_null(&self) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![],
-                            operator: "IS NOT NULL",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn between(&self, from: #rust_types, to: #rust_types) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vec![Box::new(from), Box::new(to)],
-                            operator: "BETWEEN",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn r#in(&self, values: Vec<#rust_types>) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        let mut vs: Vec<Box<dyn ToSqlValue>> = vec![];
-                        for v in values {
-                            vs.push(Box::new(v));
-                        }
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vs,
-                            operator: "IN",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-
-                    pub fn not_in(&self, values: Vec<#rust_types>) -> #builder_ident {
-                        let mut builder = self.builder.clone();
-                        let mut vs: Vec<Box<dyn ToSqlValue>> = vec![];
-                        for v in values {
-                            vs.push(Box::new(v));
-                        }
-                        builder.filters.push(Filter {
-                            table: builder.table_name_as.as_ref()
-                                .unwrap_or(&#table_name.to_string()).to_string(),
-                            name: stringify!(#column_names).to_string(),
-                            values: vs,
-                            operator: "NOT IN",
-                            preload: builder.preload,
-                        });
-                        builder
-                    }
-                }
-            )*
-
             #order_part
         };
         Ok((output_plain, output_impl, enums))
@@ -701,7 +561,7 @@ fn compute_type(
         ("uuid", _) => quote!(uuid::Uuid),
         ("USER-DEFINED", "geography(Point,4326)") => quote!(arysn::Point),
         ("USER-DEFINED", _) => {
-            let name = format_ident!("{}", udt_name.to_title_case().replace(" ", ""));
+            let name = format_ident!("{}", crate::utils::title_case(udt_name));
             quote!(#name)
         }
         _ => panic!("unknown sql type: {}", data_type),
